#!/usr/bin/env python

import sys
import os
from argparse import ArgumentParser
from settings import logger
from launchpad import LaunchpadTrigger
from launchpad import LaunchpadVote
from updateMergeProposal import build_state
from mergeproposalreview import MergeProposalReview

from branch import Branch
from tarmac.exceptions import TarmacMergeError, BranchHasConflicts

def get_fixes_commit_message(mp):
    ret = ''
    for b in mp.source_branch.linked_bugs_collection:
        ret = ret + ', ' + b.web_link
    ret = ret.replace(', ','',1)
    if ret == '':
        return ''
    else:
        return ' Fixes: ' + ret + '.'

def get_approved_by_commit_message(mp):
    ret = ''
    for vote in mp.votes:
        if vote.comment and vote.comment.vote == LaunchpadVote.APPROVE:
            ret=ret + ', ' + vote.reviewer.display_name
    ret = ret.replace(', ','',1)
    return ret



def format_commit_message(mp):
    return '%(commit_message)s.%(fixes)s Approved by %(approved_by)s.'  % {
                    'commit_message': mp.commit_message,
                    'fixes': get_fixes_commit_message(mp),
                    'approved_by': get_approved_by_commit_message(mp),
                    }

def dput(ppa, packaging_branch, revision, trunk, debfullname, 
         debemail, distributions):
    os.putenv('ppa_target',str(ppa))
    os.putenv('packaging_branch',str(packaging_branch))
    os.putenv('merge_revision',str(revision))
    os.putenv('trunk',str(trunk))
    os.putenv('DEBFULLNAME',debfullname)
    os.putenv('DEBEMAIL',debemail)
    os.putenv('distributions',distributions)
    logger.debug(('Going to execute %(dput)s with ppa_target=%(ppa_target)s, ' +
                 'packaging_branch=%(packaging_branch)s, merge_revision= ' +
                 '%(merge_revision)s, trunk=%(trunk)s, ' + 
                 'DEBFULLNAME=%(debfullname)s, DEBEMAIL=%(debemail)s, ' + 
                 'distributions=%(distributions)s') % {
                         'dput': sys.path[0] + '/dput.sh',
                         'ppa_target': str(ppa),
                         'packaging_branch': str(packaging_branch),
                         'merge_revision': str(revision),
                         'trunk': str(trunk),
                         'debfullname' : debfullname,
                         'debemail' : debemail,
                         'distributions' : distributions,
                         })
    os.system(sys.path[0]+'/dput.sh')


def change_mp_status(launchpad, mp, reason, args, vote):
    args[reason] = True
    mp_state, message, subject = build_state(args)
    message = message + args['build_job_url']
    launchpad.change_mp_status(mp, mp_state, message, subject, 
                              args['revision'], vote)

def merge_and_commit(launchpad, mp, args):
    #merging starts here
    target = Branch.create(mp.target_branch, config=False, create_tree=True)
    source = Branch.create(mp.source_branch, config=False, target=target)
    approved = source.bzr_branch.revision_id_to_revno(
                                     str(mp.reviewed_revid))

    #if unapproved changes made after approval do not land
    if source.bzr_branch.revno() > approved:
        logger.debug('Unapproved changes made after approval. ' +
                      'Failing autolanding.')
        change_mp_status(launchpad, mp, 'unapproved_changes', 
                         args, LaunchpadVote.NEEDS_FIXING)
        return 1

    try:
        logger.debug('Merging %(source)s to %(target)s' % {
                'source': str(mp.source_branch.bzr_identity),
                'target': str(mp.target_branch.bzr_identity)
                })
        target.merge(source, str(mp.reviewed_revid))
        reviews = launchpad.get_reviews(mp)
        logger.debug('Commiting previous merge')
        target.commit(format_commit_message(mp), 
                      revprops={'merge_url': mp.web_link},
                      authors=source.authors,
                      reviews=reviews)
        revision = target.bzr_branch.revno()
        logger.debug('New revision is: '+str(revision))
        if args['ppa']:
            dput(args['ppa'],args['packaging_branch'],
                            revision,target.branch_url,
                            debfullname='Martin Mrazik QA',
                            debemail='martin.mrazik+qa@canonical.com',
                            distributions=args['ppa_distributions']
                            )
        launchpad.close_bugs(mp)
        review = MergeProposalReview(mp)
        review.unlock()
    except BranchHasConflicts:
        logger.debug('Merging failed - branch has conflicts')
        change_mp_status(launchpad, mp, 'merge_failed', args, 
                         LaunchpadVote.NEEDS_FIXING)
        target.cleanup()
        return 1
    except TarmacMergeError:    
        logger.debug('Landing failed')
        change_mp_status(launchpad, mp, 'landing_failed', args, 
                         LaunchpadVote.NEEDS_FIXING)
        target.cleanup()
        return 1

    return 0

def main():
    parser = ArgumentParser(
     description="Start autolanding process resulting from a merge proposal.") 
    parser.add_argument('-p','--ppa',
                        help="PPA where the merged result shuld be dput-ed")
    parser.add_argument('-a','--packaging-branch',
                        help="Packaging branch to be used for dput-ing")
    parser.add_argument('-d','--ppa-distributions',default='',
                        help="Comma separated list of distributions " + 
                        "(e.g. precise,quantal)")
    parser.add_argument('-r','--test-result', required=True,
                        help="Result of previous testing. "+
                        "Autolanding will be stopped if this is FAILED")
    parser.add_argument('-v','--revision', required=True,
                        help="Revision of source branch that should be merged.")
    parser.add_argument('-s','--source-branch',
                        help='Branch which is supposed to be merged')
    parser.add_argument('-m','--merge-proposal',required=True,
                        help="Merge proposal which is supposed to be merged")
    parser.add_argument('-b','--build-job-url',
                        help="URL to the build/test job which was "+
                             "previously executed")
    args = vars(parser.parse_args())
    if ((args['ppa'] or args['packaging_branch']) and
         (not args['ppa'] or not args['packaging_branch'])):
            parser.error(
                 'You must specify both -p/--ppa and -a/--packaging-branch')

    launchpad = LaunchpadTrigger()

    if not args['source_branch']:
        args['source_branch'] = launchpad.get_branch_from_mp(
                                     args['merge_proposal'])
        if not args['source_branch']:
            parser.error('Unable to get branch from merge proposal.\n'+
                         'Please specify manually using --source-branch.')
   
    args['build_job_url'] = launchpad.hide_jenkins_url(args['build_job_url'])
    mp = launchpad.get_merge_proposal(args['source_branch'],
                                      args['merge_proposal'])         

    #check for test_result and fail if previous testing failed
    if args['test_result'] == 'FAILED':
        logger.debug('Previous test failed. Failing autolanding.')            
        change_mp_status(launchpad, mp, 'landing_failed', args, 
                         LaunchpadVote.NEEDS_FIXING)
        return 1

    #check for commit message and fail if it is empty
    if not (mp.commit_message):
        logger.debug('Commit message empty. Failing autolanding.')
        change_mp_status(launchpad, mp, 'commit_message_empty', 
                         args, LaunchpadVote.NEEDS_FIXING)
        return 1

    #merging starts here
    return merge_and_commit(launchpad, mp, args)

if __name__ == "__main__":
    logger.debug("Starting autolanding")            
    sys.exit(main())
