#!/usr/bin/env python

import sys
from argparse import ArgumentParser
from lockfile import FileLock
from settings import LOCK_FILE, JENKINS_URL, logger
from launchpad import LaunchpadTrigger


def trigger_ci_building(branch,job, jenkins_url):
    """Trigger a CI build."""

    launchpad_trigger = LaunchpadTrigger()
    mps = launchpad_trigger.get_merge_proposals(branch, ['Needs review'])
    launchpad_trigger.trigger_ci_building(mps, job, jenkins_url)


def trigger_al_building(branch, job, jenkins_url):
    """Trigger an autolanding build."""

    launchpad_trigger = LaunchpadTrigger()
    mps = launchpad_trigger.get_merge_proposals(branch, ['Approved'])
    launchpad_trigger.trigger_al_building(mps, job, jenkins_url)


def trigger_jenkins(branch,
                autoland=False,
                trigger_ci=False,
                job=None,
                jenkins_url=None):
    """Trigger a Jenkins build."""

    if jenkins_url is None:
        jenkins_url = JENKINS_URL

    if trigger_ci:
        trigger_ci_building(branch, job, jenkins_url)
    elif autoland:
        trigger_al_building(branch, job, jenkins_url)
    return 1


def main():

    parser = ArgumentParser(
        description="Trigger a Jenkins job resulting from a merge proposal.")
    parser.add_argument('-b', '--branch', required=True,
                        help="the proposed Launchpad branch to merge")
    parser.add_argument('-t', '--trigger-ci',
                        action='store_true',
                        help="trigger a 'CI' build")
    parser.add_argument('-l', '--autoland',
                        action='store_true',
                        help="trigger an autolanding build")
    parser.add_argument('-j', '--job', required=True,
                        help="name of the Jenkins job to trigger")
    parser.add_argument('-e', '--jenkins-url', 
                        help="URL of the Jenkins job to trigger")
    args = vars(parser.parse_args())

    if not args['trigger_ci'] and not args['autoland']:
        parser.error(
            'You need to specify one of --trigger-ci (-t) or --autoland (-l)')

    return trigger_jenkins(**args)
    

if __name__ == "__main__":
    logger.debug('Going to acquire launchpad lock')
    with FileLock(LOCK_FILE):
        logger.debug('Lock acquired')
        sys.exit(main())
