import unittest
from mock import patch
from mock import MagicMock
import tests.util as util
import autoland 
from tarmac.exceptions import TarmacMergeError, BranchHasConflicts


class TestLaunchpadTrigger(unittest.TestCase):
    def setUp(self):
        self.mp = MagicMock()
        self.LaunchpadTrigger = MagicMock()
        self.LaunchpadTrigger.get_merge_proposal = MagicMock(return_value=self.mp)
        self.launchpad_trigger_patch = patch('autoland.LaunchpadTrigger',
                         new=MagicMock(return_value=self.LaunchpadTrigger))
        self.launchpad_trigger_patch.start()
    
    def tearDown(self):
        self.launchpad_trigger_patch.stop()


class TestAutolandForMergeAndCommit(TestLaunchpadTrigger):
    def setUp(self):
        super(TestAutolandForMergeAndCommit,self).setUp()

        target = MagicMock()
        target.bzr_branch.revision_id_to_revno = MagicMock(return_value=100)
        target.bzr_branch.revno = MagicMock(return_value=50)
        branch = MagicMock()
        branch.create = MagicMock(return_value=target)
        self.branch_patch = patch('autoland.Branch',new=branch)
        self.target = target
        self.Branch = self.branch_patch.start()
        self.MergeProposalReview = MagicMock()
        self.merge_proposal_review_patch = patch(
                        'autoland.MergeProposalReview',
                        new=MagicMock(return_value=self.MergeProposalReview))
        self.merge_proposal_review_patch.start()

    def tearDown(self):
        super(TestAutolandForMergeAndCommit,self).tearDown()
        self.branch_patch.stop()
        self.merge_proposal_review_patch.stop()


class TestAutolandMergeAndCommit(TestAutolandForMergeAndCommit):
    def test_merge_and_commit_push_error(self):
        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-m','url']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            self.assertTrue(autoland.main() == 0)
            self.assertTrue(self.target.merge.call_count == 1)
            self.assertTrue(self.target.commit.call_count == 1)
            self.assertTrue(self.LaunchpadTrigger.close_bugs.call_count == 1)
            self.MergeProposalReview.unlock.assert_called_once_with()

class TestAutolandMergeCommitAndDput(TestAutolandForMergeAndCommit):
    def setUp(self):
        super(TestAutolandMergeCommitAndDput,self).setUp()
        self.os_patch = patch('autoland.os')
        self.os = self.os_patch.start()

    def tearDown(self):
        super(TestAutolandMergeCommitAndDput,self).tearDown()
        self.os_patch.stop()

    def test_merge_and_commit_push_error(self):
        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-m','url', '-p', 'ppa:myppa', 
                    '-a', 'packaging_branch']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            self.assertTrue(autoland.main() == 0)
            self.assertTrue(self.target.merge.call_count == 1)
            self.assertTrue(self.target.commit.call_count == 1)
            self.assertTrue(self.LaunchpadTrigger.close_bugs.call_count == 1)
            self.MergeProposalReview.unlock.assert_called_once_with()
            self.assertTrue(self.os.system.call_count == 1)
            self.assertTrue(self.os.putenv.called)


class TestAutolandMergeAndCommitMergeConflict(TestLaunchpadTrigger):
    def setUp(self):
        super(TestAutolandMergeAndCommitMergeConflict,self).setUp()

        source = MagicMock()
        source.bzr_branch.revision_id_to_revno = MagicMock(return_value=100)
        source.bzr_branch.revno = MagicMock(return_value=50)
        source.merge.side_effect = BranchHasConflicts('branchHasConflicts')
        branch = MagicMock()
        branch.create = MagicMock(return_value=source)
        self.branch_patch = patch('autoland.Branch',new=branch)
        self.Branch = self.branch_patch.start()

    def tearDown(self):
        super(TestAutolandMergeAndCommitMergeConflict,self).tearDown()
        self.branch_patch.stop()

    def test_merge_and_commit_merge_conflict(self):
        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-m','url']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            self.assertTrue(autoland.main() == 1)


class TestAutolandMergeAndCommitPushFailed(TestLaunchpadTrigger):
    def setUp(self):
        super(TestAutolandMergeAndCommitPushFailed, self).setUp()

        source = MagicMock()
        source.bzr_branch.revision_id_to_revno = MagicMock(return_value=100)
        source.bzr_branch.revno = MagicMock(return_value=50)
        source.merge.side_effect = TarmacMergeError('tarmacMergeError')
        branch = MagicMock()
        branch.create = MagicMock(return_value=source)
        self.branch_patch = patch('autoland.Branch',new=branch)
        self.Branch = self.branch_patch.start()

    def tearDown(self):
        super(TestAutolandMergeAndCommitPushFailed, self).tearDown()
        self.branch_patch.stop()

    def test_merge_and_commit_push_error(self):
        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-m','url']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            self.assertTrue(autoland.main() == 1)

class TestAutolandMergeAndCommitUnapprovedChange(TestLaunchpadTrigger):
    def setUp(self):
        super(TestAutolandMergeAndCommitUnapprovedChange, self).setUp()

        source = MagicMock()
        source.bzr_branch.revision_id_to_revno = MagicMock(return_value=100)
        source.bzr_branch.revno = MagicMock(return_value=500)
        branch = MagicMock()
        branch.create = MagicMock(return_value=source)
        self.branch_patch = patch('autoland.Branch',new=branch)
        self.Branch = self.branch_patch.start()

    def tearDown(self):
        super(TestAutolandMergeAndCommitUnapprovedChange, self).tearDown()
        self.branch_patch.stop()

    def test_merge_and_commit_unapproved_change(self):
        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-m','url']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            self.assertTrue(autoland.main() == 1)
            self.assertTrue(
                   self.LaunchpadTrigger.change_mp_status.call_count == 1)


class TestAutolandEmptyCommitMessage(TestLaunchpadTrigger):

    def setUp(self):
        super(TestAutolandEmptyCommitMessage, self).setUp()
        self.mp.commit_message = ''

    def test_empty_commit_message(self):
        """Test the 'empty commit message' error."""

        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-m','url']
        error_message = ('Unable to get branch from merge proposal.\n'+
		                 'Please specify manually using --source-branch.')
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            self.assertTrue(autoland.main() == 1)
            self.assertTrue(
                self.LaunchpadTrigger.change_mp_status.call_count == 1)

class TestAutolandSourceBranchError(unittest.TestCase):

    def setUp(self):
        launchpadTrigger = MagicMock()
        launchpadTrigger.get_branch_from_mp = MagicMock(return_value=None)
        self.launchpad_trigger_patch = patch('autoland.LaunchpadTrigger',
                         new=MagicMock(return_value=launchpadTrigger))
        self.launchpad_trigger_patch.start()

    def tearDown(self):
        self.launchpad_trigger_patch.stop()

    def test_merge_proposal_without_branch(self):
        """Test the 'Unable to get branch from merge proposal' error."""

        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-m','url']
        error_message = ('Unable to get branch from merge proposal.\n'+
		                 'Please specify manually using --source-branch.')
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                autoland.main()
                self.assertTrue(False)
            except SystemExit:
                pass
            self.assertTrue(error_message in stderr.getvalue())


class TestAutoland(unittest.TestCase):

    def setUp(self):
        self.LaunchpadTrigger = MagicMock()
        self.LaunchpadTrigger.hide_jenkins_url = MagicMock(return_value='url')
        self.launchpad_trigger_patch = patch('autoland.LaunchpadTrigger',
                              new=MagicMock(return_value=self.LaunchpadTrigger))
        self.launchpad_trigger_patch.start()

    def tearDown(self):
        self.launchpad_trigger_patch.stop()


    def test_gibberish(self):
        """Gibberish on the command line produces usage message."""

        sys_argv = ['--oheck']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                autoland.main()
            except SystemExit:
                pass
            self.assertTrue("usage" in stderr.getvalue())


    def test_ppa_specified_but_no_packaging_branch(self):
        """Generate the 'specify both -p and -a' error."""

        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-p', 'ppa:asdf', '-m','url']
        error_message = ('You must specify both -p/--ppa '+
                          'and -a/--packaging-branch')
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                autoland.main()
                self.assertTrue(False)
            except SystemExit:
                pass
            self.assertTrue(error_message in stderr.getvalue())

    def test_packaging_branch_specified_but_no_ppa(self):
        """Generate the 'specify both -p and -a' error."""

        sys_argv = ['autoland.py', '-r', 'PASSED',
                    '-v', '123', '-a', 'url', '-m','url']
        error_message = ('You must specify both -p/--ppa '+
                          'and -a/--packaging-branch')
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                autoland.main()
                self.assertTrue(False)
            except SystemExit:
                pass
            self.assertTrue(error_message in stderr.getvalue())
            
    def test_test_result_failed(self):
        """Fail autolanding because of failed testing"""

        sys_argv = ['autoland.py', '-r', 'FAILED',
                    '-v', '123', '-m','url']
        error_message = ('You must specify both -p/--ppa '+
                          'and -a/--packaging-branch')
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
                autoland.main()
                self.assertTrue(
                    self.LaunchpadTrigger.change_mp_status.call_count == 1)

