import sys
import unittest
import launchpad
from settings import *
from mock import *



class LaunchpadAgentTestCase(unittest.TestCase):
    launchpadAgent = None

    def setUp(self):
        l = MagicMock(name="launchpadlib.launchpadMock")
        with patch('launchpad.Launchpad',new=l):
            self.launchpadAgent = launchpad.LaunchpadAgent()

    def test_change_mp_status_with_no_vote(self):
        mp = MagicMock()
        self.launchpadAgent.change_mp_status(mp, 'Status', 'Message', 
                        'Subject', revision=1)
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        subject='Subject',
                        content='Message')
        mp.setStatus.assert_called_once_with(status='Status',revid=1)

    def test_change_mp_status_with_approve(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        self.launchpadAgent.change_mp_status(mp, 'Status', 'Message', 
                        'Subject', revision=1,vote='Approve')
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        subject='Subject',
                        content='Message',vote='Approve')
        mp.setStatus.assert_called_once_with(status='Status',revid=1)

    def test_change_mp_status_with_disapprove(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        self.launchpadAgent.change_mp_status(mp, 'Status', 'Message', 
                        'Subject', revision=1,vote='Disapprove')
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        subject='Subject',
                        content='Message',vote='Disapprove')
        mp.setStatus.assert_called_once_with(status='Status',revid=1)

    def test_approve_mp(self):
        mp = MagicMock()
        mp.web_link = 'http://mp'
        self.launchpadAgent.approve_mp(mp, '1','http://url')
        subject = 'PASSED: Continuous integration, rev:1'
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        vote=launchpad.LaunchpadVote.APPROVE,
                        subject=subject,
                        content=subject+"\nhttp://url")

    def test_disapprove_mp(self):
        mp = MagicMock()
        mp.web_link = 'http://mp'
        self.launchpadAgent.disapprove_mp(mp, '1','http://url')
        subject = 'FAILED: Continuous integration, rev:1'
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        vote=launchpad.LaunchpadVote.NEEDS_FIXING,
                        subject=subject,
                        content=subject+"\nhttp://url")

    def test_is_job_published_with_private_job(self):
        jenkins = MagicMock()
        jenkins.get_job_config = MagicMock(return_value='<b></b>')
        with patch('launchpad.jenkins.Jenkins', 
                   new=MagicMock(return_value=jenkins)):
            self.assertFalse(self.launchpadAgent.is_job_published('my-job'))

    def test_is_job_published_with_no_job(self):
        jenkins = MagicMock()
        jenkins.get_job_config = MagicMock(return_value='')
        with patch('launchpad.jenkins.Jenkins', 
                   new=MagicMock(return_value=jenkins)):
            self.assertFalse(self.launchpadAgent.is_job_published(None))

    def test_is_job_published_with_public_job(self):
        jenkins = MagicMock()
        jenkins.get_job_config = MagicMock(return_value=
                        '<hudson.plugins.build__publisher.BuildPublisher>'+
                        '</hudson.plugins.build__publisher.BuildPublisher>'
                        )
        with patch('launchpad.jenkins.Jenkins', 
                   new=MagicMock(return_value=jenkins)):
            self.assertTrue(self.launchpadAgent.is_job_published('my-job'))

    def test_is_job_published_with_exception(self):
        jenkins = MagicMock()
        jenkins.get_job_config.side_effect = Exception('get_job_config exception')
        with patch('launchpad.jenkins.Jenkins', 
                   new=MagicMock(return_value=jenkins)):
            self.assertFalse(self.launchpadAgent.is_job_published('my-job'))

class LaunchpadAgentTestHideJenkinsUrl(unittest.TestCase):
    launchpadAgent = None

    def setUp(self):
        l = MagicMock(name="launchpadlib.launchpadMock")
        with patch('launchpad.Launchpad',new=l):
            self.launchpadAgent = launchpad.LaunchpadAgent()


    def test_hide_jenkins_url_public_url(self):
        urls = [
                ('http://10.0.0.1:8080/job/android-audiosystem/31/',
                 PUBLIC_JENKINS_URL+'/job/android-audiosystem/31/'),
                ('http://10.0.0.1:8080/job/android-ubuntu-matrix/4/',
                 PUBLIC_JENKINS_URL+'/job/android-ubuntu-matrix/4/'),
                ('https://10.0.0.1:8080/job/unity-ci/32/',
                 PUBLIC_JENKINS_URL+'/job/unity-ci/32/')
               ]

        launchpad_agent = launchpad.LaunchpadAgent
        launchpad_agent.is_job_published = staticmethod(lambda job : True)
        for url, expected in urls:
            with patch('launchpad.LaunchpadAgent',new=launchpad_agent):
                self.assertEqual(self.launchpadAgent.hide_jenkins_url(url),
                                 expected)
    
    def test_hide_jenkins_url_hidden_url(self):
        urls = [
                'https://10.0.0.1:8080/job/android-audiosystem/31/',
                'http://10.0.0.1:8080/job/android-ubuntu-matrix/4/',
                'http://10.0.0.1:8080/job/unity-ci/32/',
               ]

        launchpad_agent = launchpad.LaunchpadAgent
        launchpad_agent.is_job_published = staticmethod(lambda job : False)
        for url in urls:
            with patch('launchpad.LaunchpadAgent',new=launchpad_agent):
                self.assertEqual(self.launchpadAgent.hide_jenkins_url(url),
                           url.replace('http://10.0.0.1:8080',
                                       'http://jenkins:8080'))

    def test_hide_jenkins_url_unknown_url(self):
        urls = [
                'http://12.0.0.2:8080/job/android-audiosystem/31/',
                'http://12.0.0.2:8080/job/android-ubuntu-matrix/4/',
                'https://12.0.0.2:8080/job/unity-ci/32/',
                'https://12.0.0.080/droid-audiosystem/31/',
                'http://12.0.0.080/droid-ubuntu-matrix/4/',
                'http://12.0.0.080/ity-ci/32/',
               ]

        launchpad_agent = launchpad.LaunchpadAgent
        launchpad_agent.is_job_published = staticmethod(lambda job : False)
        for url in urls:
            with patch('launchpad.LaunchpadAgent',new=launchpad_agent):
                self.assertEqual(self.launchpadAgent.hide_jenkins_url(url),
                                 url)
