import sys
import unittest
import launchpad
from settings import *
from mock import *
import launchpadTrigger
import jenkins


class LaunchpadTriggerInstantiation(unittest.TestCase):
    def test_instatiation(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            l = launchpad.LaunchpadTrigger()
            l._launchpad.people.assert_called_once_with(LAUNCHPAD_LOGIN)


class LaunchpadTriggerTestCase(unittest.TestCase):

    launchpadTrigger = None
    jenkinsUserName = 'jenkins-user'

    def setUp(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        user = MagicMock()
        user.name = self.jenkinsUserName
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            self.launchpadTrigger = launchpad.LaunchpadTrigger()
            self.launchpadTrigger._launchpad_user = user


    def test_new_mp_candidate_revision_in_queue(self):
            #60 is the revision which was validated as last
            self.launchpadTrigger.get_latest_review = MagicMock(return_value=60)
            mp = MagicMock()
            #this is the current branch revision
            mp.source_branch.revision_count = 61
            #latest_candidate_validated should return False as there is a new
            #revision in the queue (61)
            self.assertFalse(self.launchpadTrigger.latest_candidate_validated(mp))

    def test_mp_already_validated(self):
            #60 is the revision which was validated as last
            self.launchpadTrigger.get_latest_review = MagicMock(return_value=60)
            mp = MagicMock()
            #this is the current branch revision
            mp.source_branch.revision_count = 60
            #latest_candidate_validated should return True. This rev was
            #already validated
            self.assertTrue(self.launchpadTrigger.latest_candidate_validated(mp))


    def test_get_latest_review_no_comments(self):
        mp = MagicMock()
        mp.all_comments = []
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 0)


    def test_get_latest_review_unrelated_comment(self):
        mp = MagicMock()
        comment = MagicMock()
        comment.vote_tag = LAUNCHPAD_REVIEW_TYPE
        comment.message_body = 'unrelated comment'
        mp.all_comments = [comment]
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 0)


    def test_get_latest_review_which_passed(self):
        mp = MagicMock()
        comment = MagicMock()
        comment.author.name = self.jenkinsUserName
        comment.vote_tag = LAUNCHPAD_REVIEW_TYPE
        comment.message_body = "PASSED: Continuous integration, rev:47\nURL:..."
        mp.all_comments = [comment]
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 47)

    def test_get_latest_review_which_failed(self):
        mp = MagicMock()
        comment = MagicMock()
        comment.author.name = self.jenkinsUserName
        comment.vote_tag = LAUNCHPAD_REVIEW_TYPE
        comment.message_body = "FAILED: Continuous integration, rev:47\nURL:..."
        mp.all_comments = [comment]
        self.assertEqual(self.launchpadTrigger.get_latest_review(mp), 47)


    def test_testing_not_in_progress_approved(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        review = launchpad.MergeProposalReview(mp)
        review.unlock()
        self.assertFalse(self.launchpadTrigger.testing_in_progress(mp))


    def test_testing_not_in_progress_abstain(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        review = launchpad.MergeProposalReview(mp)
        review.lock()
        self.assertTrue(self.launchpadTrigger.testing_in_progress(mp))

    def test_user_not_allowed_to_trigger_job(self):
        mp = MagicMock()
        mp.registrant.name = 'random-user'
        membership = MagicMock()
        membership.team.name = 'random-team'
        mp.registrant.memberships_details = [membership]
        self.assertFalse(self.launchpadTrigger.user_allowed_to_trigger_jobs(mp))


    def test_user_member_of_allowed_team(self):
        mp = MagicMock()
        mp.registrant.name = 'random-user'
        membership = MagicMock()
        membership.team.name = 'mrazik'
        mp.registrant.memberships_details = [membership]
        self.assertTrue(self.launchpadTrigger.user_allowed_to_trigger_jobs(mp))

    def test_user_allowed_to_trigger(self):
        mp = MagicMock()
        mp.registrant.name = 'mrazik'
        membership = MagicMock()
        membership.team.name = ''
        mp.registrant.memberships_details = [membership]
        self.assertTrue(self.launchpadTrigger.user_allowed_to_trigger_jobs(mp))

    def test_start_jenkins_job(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        j = MagicMock()
        j.get_job_info = MagicMock(return_value = {'buildable' : True})
        jenkins = MagicMock()
        jenkins.Jenkins = MagicMock(return_value = j)
        with patch('launchpad.jenkins',new=jenkins):
            self.launchpadTrigger.start_jenkins_job('url','jenkins_job',mp)
            self.assertEqual(j.build_job.call_count,1)
            self.assertEqual(mp.nominateReviewer.call_count,1)

    def test_start_jenkins_job_and_raise_exception(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        j = MagicMock()
        j.get_job_info = MagicMock(return_value = {'buildable' : True})
        j.build_job.side_effect = jenkins.JenkinsException("jenkinsException")
        jenkinsInstance = MagicMock()
        jenkinsInstance.Jenkins = MagicMock(return_value = j)
        jenkinsInstance.JenkinsException = jenkins.JenkinsException
        with patch('launchpad.jenkins',new=jenkinsInstance):
            self.launchpadTrigger.start_jenkins_job('url','jenkins_job',mp)
            self.assertEqual(mp.createComment.call_count, 1)



    def test_start_nonbuildable_jenkins_job(self):
        mp = MagicMock()
        j = MagicMock()
        j.get_job_info = MagicMock(return_value = {'buildable' : False})
        jenkins = MagicMock()
        jenkins.Jenkins = MagicMock(return_value = j)
        with patch('launchpad.jenkins',new=jenkins):
            self.assertEqual(self.launchpadTrigger.start_jenkins_job('url','jenkins_job',mp), False)
            self.assertEqual(j.build_job.call_count,0)
            self.assertEqual(mp.nominateReviewer.call_count,0)
            self.assertEqual(mp.createComment.call_count,0)

    def test_unapproved_prerequisite_no_prerequisite(self):
        mp = MagicMock()
        mp.prerequisite_branch = None
        self.assertFalse(self.launchpadTrigger.unapproved_prerequisite_exists(mp))

    def test_unapproved_prerequisite_too_many_proposals(self):
        mp = MagicMock()
        mp.target_branch = MagicMock()
        mp.target_branch.web_link = 'url'
        prereq = MagicMock()
        prereq.landing_targets = [MagicMock(), MagicMock(), MagicMock()]
        for t in prereq.landing_targets:
                t.target_branch.web_link = 'url'
                t.queue_status = 'Approved'
        mp.prerequisite_branch = prereq
        self.assertTrue(
             self.launchpadTrigger.unapproved_prerequisite_exists(mp))

    def test_unapproved_prerequisite_no_proposal_for_prerequisite(self):
        mp = MagicMock()
        prereq = MagicMock()
        prereq.landing_targets = []
        mp.prerequisite_branch = prereq
        self.assertTrue(
             self.launchpadTrigger.unapproved_prerequisite_exists(mp))

    def test_unapproved_prerequisite_not_merged_yet(self):
        mp = MagicMock()
        mp.target_branch = MagicMock()
        mp.target_branch.web_link = 'url'
        prereq = MagicMock()
        prereq.landing_targets = [MagicMock()]
        for t in prereq.landing_targets:
                t.target_branch.web_link = 'url'
                t.queue_status = 'Approved'
        mp.prerequisite_branch = prereq
        self.assertTrue(
             self.launchpadTrigger.unapproved_prerequisite_exists(mp))

    def test_unapproved_prerequisite_merged(self):
        mp = MagicMock()
        mp.target_branch = MagicMock()
        mp.target_branch.web_link = 'url'
        prereq = MagicMock()
        prereq.landing_targets = [MagicMock()]
        for t in prereq.landing_targets:
                t.target_branch.web_link = 'url'
                t.queue_status = 'Merged'
        mp.prerequisite_branch = prereq
        self.assertFalse(
             self.launchpadTrigger.unapproved_prerequisite_exists(mp))

    def test_get_reviews_no_reviews(self):
        mp = MagicMock()
        mp.votes = []
        self.assertTrue(self.launchpadTrigger.get_reviews(mp) is None)

    def test_get_reviews(self):
        mp = MagicMock()
        mp.votes = [MagicMock(), MagicMock(), MagicMock()]
        for vote in mp.votes:
            vote.reviewer.display_name = 'jenkins'
            vote.comment.vote = 'Approve'
        reviews = self.launchpadTrigger.get_reviews(mp)
        for r in reviews:
            self.assertTrue(r == 'jenkins;Approve')               

    def test_close_bugs(self):
        mp = MagicMock()
        mp.target_branch.project.name = 'jenkins'
        mp.source_branch.linked_bugs_collection = [MagicMock(), MagicMock()]
        for b in mp.source_branch.linked_bugs_collection:
            b.bug_tasks_collection = [MagicMock(), MagicMock(), MagicMock()]
            for task in b.bug_tasks_collection:
                    task.target.name = 'jenkins'

        self.launchpadTrigger.close_bugs(mp)
        for b in mp.source_branch.linked_bugs_collection:
            for task in b.bug_tasks_collection:
                    task.lp_save.assert_called_once_with()

class TestLaunchpadTrigger(unittest.TestCase):
    def setUp(self):
        launchpadAgent = MagicMock(name="launchpadAgentMock")
        launchpadAgent.login = MagicMock(return_value=MagicMock())
        with patch('launchpad.LaunchpadAgent',new=launchpadAgent):
            self.launchpadTrigger = launchpad.LaunchpadTrigger()

class LaunchpadTriggerCIWithWrongUser(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerCIWithWrongUser, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock(return_value=False)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_with_wrong_user(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)

    def test_trigger_ci_with_no_mps(self):
         self.launchpadTrigger.trigger_ci_building([],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerCIWhileTestingInProgress(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerCIWhileTestingInProgress, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock()
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=True)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_while_testing_in_progress(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerCIWhileRevisionValidated(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerCIWhileRevisionValidated, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock()
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=False)
        self.launchpadTrigger.latest_candidate_validated = MagicMock(return_value=True)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_while_Revision_validated(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerCIW(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerCIW, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock()
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=False)
        self.launchpadTrigger.latest_candidate_validated = MagicMock(return_value=False)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_ci_while_Revision_validated(self):
         self.launchpadTrigger.trigger_ci_building([MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,1)


class LaunchpadTriggerAutolandingWithoutPermissions(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerAutolandingWithoutPermissions, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock(return_value=False)
        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_al_without_permissions(self):
         self.launchpadTrigger.trigger_al_building(
                         [MagicMock(),MagicMock(),MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerAutolandingWhileTestingInProgress(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerAutolandingWhileTestingInProgress, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock(return_value=True)
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=True)

        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_al_while_testing_in_progress(self):
         self.launchpadTrigger.trigger_al_building(
                         [MagicMock(),MagicMock(),MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)


class LaunchpadTriggerAutolandingWithUnapprovedPrerequisite(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerAutolandingWithUnapprovedPrerequisite, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock(return_value=True)
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=False)
        self.launchpadTrigger.unapproved_prerequisite_exists = MagicMock(return_value=True)

        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_al_with_unapproved_prerequisite(self):
         self.launchpadTrigger.trigger_al_building(
                         [MagicMock(),MagicMock(),MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,0)

class LaunchpadTriggerAutolanding(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerAutolanding, self).setUp()
        self.launchpadTrigger.user_allowed_to_trigger_jobs = MagicMock(return_value=True)
        self.launchpadTrigger.testing_in_progress = MagicMock(return_value=False)
        self.launchpadTrigger.unapproved_prerequisite_exists = MagicMock(return_value=False)

        self.launchpadTrigger.start_jenkins_job = MagicMock()

    def test_trigger_al_with_unapproved_prerequisite(self):
         self.launchpadTrigger.trigger_al_building(
                         [MagicMock(),MagicMock(),MagicMock()],'job','url')
         self.assertEqual(self.launchpadTrigger.start_jenkins_job.call_count,3)

class LaunchpadTriggerTestGetMergeProposalsWithInvalidBranch(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerTestGetMergeProposalsWithInvalidBranch, self).setUp()
        self.launchpadTrigger._launchpad = MagicMock()
        self.launchpadTrigger._launchpad.branches.getByUrl = MagicMock(return_value = None)

    def test_get_merge_proposals_with_invalid_branch(self):
         self.assertEqual(self.launchpadTrigger.get_merge_proposals('',''),[])

class LaunchpadTriggerTestGetMergeProposals(TestLaunchpadTrigger):
    def setUp(self):
        super(LaunchpadTriggerTestGetMergeProposals, self).setUp()
        self.launchpadTrigger._launchpad = MagicMock()
        branch = MagicMock()
        branch.landing_candidates = []
        for i in range(1,10):
            mp = MagicMock()
            mp.queue_status = i
            branch.landing_candidates.append(mp)
        self.launchpadTrigger._launchpad.branches.getByUrl = MagicMock(return_value = branch)

    def test_get_merge_proposals_odd(self):
        mps = self.launchpadTrigger.get_merge_proposals('',[1,3,5,7])
        self.assertEqual(len(mps),4) 

    def test_get_merge_proposal_two(self):
        mps = self.launchpadTrigger.get_merge_proposals('',[2])
        self.assertEqual(len(mps),1) 
        self.assertEqual(mps[0].queue_status,2) 

