import unittest
from mock import patch
import tests.util as util
import launchpadTrigger


class TestLaunchpadTriggerParams(unittest.TestCase):

    merge_proposal = \
        'https://code.launchpad.net/~mrazik/faux-dbus-test-runner/yar8/+merge/106113'

    def setUp(self):
        self.launchpad_trigger_patch = patch('launchpadTrigger.LaunchpadTrigger')
        self.LaunchpadTrigger = self.launchpad_trigger_patch.start()


    def tearDown(self):
        self.launchpad_trigger_patch.stop()


    def test_gibberish(self):
        """Gibberish on the command line produces usage message."""

        sys_argv = ['--oheck']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                launchpadTrigger.main()
            except SystemExit:
                pass
            self.assertTrue("usage" in stderr.getvalue())


    def test_no_trigger_ci_nor_autoland_specified(self):
        """Generate the 'specify either -t or -l' error."""

        sys_argv = ['launchpadTrigger.py', '-b', 'lp:faux-dbus-test-runner',
                    '-j', 'test-job', '-e', 'jenkins-url']
        error_message = 'You need to specify one of --trigger-ci (-t)'+\
                        ' or --autoland (-l)'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr, \
                patch('launchpadTrigger.sys.exit') as sys_exit:
            launchpadTrigger.main()
            sys_exit.assert_called_with(2)
            self.assertTrue(error_message in stderr.getvalue())


    def test_no_branch_specified(self):
        """Failed to specify branch produces error."""

        sys_argv = ["launchpadTrigger.py",
                    "-a",
                    "--job=faux-dbus-test-runner",
                    "--autoland"]
        error_message = "error: argument -b/--branch is required"
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                launchpadTrigger.main()
            except SystemExit:
                pass
            self.assertTrue(error_message in stderr.getvalue())

    def test_trigger_autolanding_no_jenkins_job(self):
        """Attempt to trigger fails with no Jenkins job."""

        sys_argv = ["launchpadTrigger.py",
                    "-a",
                    "-n",
                    "--branch=lp:faux-dbus-test-runner",
                    "--autoland"]
        error_message = 'argument -j/--job is required'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                launchpadTrigger.main()
            except SystemExit:
                pass
            self.assertTrue(error_message in stderr.getvalue())


    def test_trigger_autolanding(self):
        """Trigger autolanding with a set of command-line params."""

        sys_argv = ["launchpadTrigger.py",
                    "--job=faux-dbus-test-runner",
                    "--branch=lp:faux-dbus-test-runner",
                    "--autoland"]
        with patch('sys.argv', sys_argv), \
                patch.object(self.LaunchpadTrigger.return_value, 'get_merge_proposals') as get_merge_proposals, \
                patch.object(self.LaunchpadTrigger.return_value, 'trigger_al_building') as trigger_al_building:
            get_merge_proposals.return_value = [self.merge_proposal]
            launchpadTrigger.main()
            trigger_al_building.assert_called_with([self.merge_proposal],
                                                   'faux-dbus-test-runner',
                                                   'http://localhost:8080/')

    
    def test_trigger_ci(self):
        """Trigger CI with a set of command-line params."""

        sys_argv = ["launchpadTrigger.py",
                    "--branch=lp:faux-dbus-test-runner",
                    "--trigger-ci",
                    "--job=faux-dbus-test-runner"]
        with patch('sys.argv', sys_argv), \
                patch.object(self.LaunchpadTrigger.return_value, 'get_merge_proposals') as get_merge_proposals, \
                patch.object(self.LaunchpadTrigger.return_value, 'trigger_ci_building') as trigger_ci_building:
            get_merge_proposals.return_value = [self.merge_proposal]
            launchpadTrigger.main()
            trigger_ci_building.assert_called_with([self.merge_proposal],
                                                   'faux-dbus-test-runner',
                                                   'http://localhost:8080/')

    def test_trigger_ci_needs_review(self):
        """Trigger CI with a set of command-line params."""

        sys_argv = ["launchpadTrigger.py",
                    "--branch=lp:faux-dbus-test-runner",
                    "--trigger-ci",
                    "--job=faux-dbus-test-runner",]
        with patch('sys.argv', sys_argv), \
                patch.object(self.LaunchpadTrigger.return_value, 'get_merge_proposals') as get_merge_proposals, \
                patch.object(self.LaunchpadTrigger.return_value, 'trigger_ci_building') as trigger_ci_building:
            get_merge_proposals.return_value = [self.merge_proposal]
            launchpadTrigger.main()
            trigger_ci_building.assert_called_with([self.merge_proposal],
                                                   'faux-dbus-test-runner',
                                                   'http://localhost:8080/')

