import unittest
from mock import patch
import tests.util as util
import voteOnMergeProposal


class TestVoteOnMergeProposal(unittest.TestCase):

    def setUp(self):
        self.launchpad_trigger_patch = patch(
                                       'voteOnMergeProposal.LaunchpadTrigger')
        self.LaunchpadTrigger = self.launchpad_trigger_patch.start()

    def tearDown(self):
        self.launchpad_trigger_patch.stop()

    def test_gibberish(self):
        """Gibberish on the command line produces usage message."""

        sys_argv = ['--oheck']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            try:
                voteOnMergeProposal.main()
            except SystemExit:
                pass
            self.assertTrue("usage" in stderr.getvalue())

    def test_no_build_url_specified(self):
        """No build_url specified; exit with message."""

        sys_argv = ['voteOnMergeProposal.py',
                    '--status=PASSED',
                    '--revision=66',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/faux-dbus-test-runner/yar8/+merge/106113"']
        error_message = 'error: argument -u/--build-url is required'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            with self.assertRaises(SystemExit) as e:
                voteOnMergeProposal.main()
            self.assertTrue(error_message in stderr.getvalue())
            self.assertEquals(2, e.exception.code)


    def test_no_revision_specified(self):
        """No revision specified; exit with message."""

        sys_argv = ['voteOnMergeProposal.py',
                    '--status=PASSED',
                    '--build-url="http://s-jenkins/job/faux-dbus-test-runner-ci/configure"',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/faux-dbus-test-runner/yar8/+merge/106113"']
        error_message = 'error: argument -r/--revision is required'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            with self.assertRaises(SystemExit) as e:
                voteOnMergeProposal.main()
            self.assertTrue(error_message in stderr.getvalue())
            self.assertEquals(2, e.exception.code)


    def test_no_merge_proposal_specified(self):
        """No merge_proposal specified; exit with message."""

        sys_argv = ['voteOnMergeProposal.py',
                    '--status=PASSED',
                    '--build-url="http://s-jenkins/job/faux-dbus-test-runner-ci/configure"',
                    '--revision=66',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"']
        error_message = 'error: argument -p/--merge-proposal is required'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr:
            with self.assertRaises(SystemExit) as e:
                voteOnMergeProposal.main()
            self.assertTrue(error_message in stderr.getvalue())
            self.assertEquals(2, e.exception.code)


    def test_no_branch_and_none_found(self):
        """No branch specified and none can be inferred."""
        
        sys_argv = ['voteOnMergeProposal.py',
                    '--status=PASSED',
                    '--build-url="http://s-jenkins/job/faux-dbus-test-runner-ci/configure"',
                    '--revision=66',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/faux-dbus-test-runner/yar8/+merge/106113"']
        error_message = 'Error: unable to get branch from merge proposal.\n\
                            Please specify manually using -- branch.'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr, \
                patch.object(self.LaunchpadTrigger.return_value, 'get_branch_from_mp') as get_branch_from_mp:
            get_branch_from_mp.return_value = None
            with self.assertRaises(SystemExit) as e:
                voteOnMergeProposal.main()
            self.assertTrue(error_message in stderr.getvalue())
            self.assertEquals(2, e.exception.code)


    def test_unknown_merge_proposal(self):
        """The given merge proposal wasn't found; error out with message."""

        sys_argv = ['voteOnMergeProposal.py',
                    '--status=PASSED',
                    '--build-url="http://s-jenkins/job/faux-dbus-test-runner-ci/configure"',
                    '--revision=66',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/faux-dbus-test-runner/yar8/+merge/106113"']
        error_message = 'error: merge proposal relted to this branch was not found'
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr, \
                patch.object(self.LaunchpadTrigger.return_value, 'get_merge_proposal') as get_merge_proposal:
            get_merge_proposal.return_value = None
            try:
                voteOnMergeProposal.main()
            except SystemExit:
                pass
            self.assertTrue(error_message in stderr.getvalue())


    def test_passed_vote(self):
        """Vote on a proposal which passed."""

        sys_argv = ['voteOnMergeProposal.py',
                    '--status=PASSED',
                    '--build-url="http://s-jenkins/job/faux-dbus-test-runner-ci/configure"',
                    '--revision=66',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/faux-dbus-test-runner/yar8/+merge/106113"']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr, \
                patch('voteOnMergeProposal.sys.exit') as sys_exit, \
                patch.object(self.LaunchpadTrigger.return_value, 'get_merge_proposal') as get_merge_proposal, \
                patch.object(self.LaunchpadTrigger.return_value, 'hide_jenkins_url') as hide_jenkins_url, \
                patch.object(self.LaunchpadTrigger.return_value, 'approve_mp') as approve_mp:
            get_merge_proposal.return_value = 'Launchpad_mp'
            hide_jenkins_url.return_value = 'http://s-jenkins/job/faux-dbus-test-runner-ci/configure'
            voteOnMergeProposal.main()
            approve_mp.assert_called_with('Launchpad_mp',
                                          '66',
                                          'http://s-jenkins/job/faux-dbus-test-runner-ci/configure')


    def test_not_passed_vote(self):
        """Vote on a proposal which did not pass."""

        sys_argv = ['voteOnMergeProposal.py',
                    '--status=OHECK',
                    '--build-url="http://s-jenkins/job/faux-dbus-test-runner-ci/configure"',
                    '--revision=66',
                    '--branch="lp:~mrazik/faux-dbus-test-runner/yar8"',
                    '--merge-proposal="https://code.launchpad.net/~mrazik/faux-dbus-test-runner/yar8/+merge/106113"']
        with patch('sys.argv', sys_argv), \
                util.captured_stderr() as stderr, \
                patch('voteOnMergeProposal.sys.exit') as sys_exit, \
                patch.object(self.LaunchpadTrigger.return_value, 'get_merge_proposal') as get_merge_proposal, \
                patch.object(self.LaunchpadTrigger.return_value, 'hide_jenkins_url') as hide_jenkins_url, \
                patch.object(self.LaunchpadTrigger.return_value, 'disapprove_mp') as disapprove_mp:
            get_merge_proposal.return_value = 'Launchpad_mp'
            hide_jenkins_url.return_value = 'http://s-jenkins/job/faux-dbus-test-runner-ci/configure'
            voteOnMergeProposal.main()
            disapprove_mp.assert_called_with('Launchpad_mp',
                                             '66',
                                             'http://s-jenkins/job/faux-dbus-test-runner-ci/configure')
